/**
 \file com/st/p2012/mind/comments/LinkTag.java
 \brief A class to represent a link tag in a comment.


 <B>
 \n
 This file is part of the Platform 2012 program,
 a cooperation between STMicroelectronics and CEA.\n
 Redistribution of this file to outside parties is
 strictly prohibited without the written consent
 of the module owner indicated below.\n
 </B>

 \par  Module owner: michel.metzger@st.com


 \par  Copyright (C) 2009 STMicroelectronics

 \par  Authors: michel.metzger@st.com

 \par  Id: $Id$
 \par  Date: $Date$
 \par  Revision: $Rev$

 */
package com.st.p2012.mind.comments;

import static com.st.p2012.mind.HTMLDocumentationHelper.getShortName;

import com.st.p2012.mind.HTMLDocumentationHelper;

public abstract class LinkTag<E extends Enum<E>> extends CommentTag {

  public static enum ComponentLinkElementType {
    INTERFACES, COMPONENTS, ATTRIBUTES;
  }

  public static enum InterfaceLinkElementType {
    METHODS,
    UNIONS,
    STRUCTS,
    ENUMS,
    TYPEDEFS,
    TYPES
  }

  protected static final String THIS_IDENTIFIER = "this";
  protected String target;
  protected final E subElemType;
  protected final String subElemName;

  public LinkTag(final String target, final String type, final String subElemName,
      final int beginIndex, final int endIndex) throws CommentParserException {
    super(beginIndex, endIndex);

    if(type != null)
      this.subElemType = parseSubElemType(type);
    else
      this.subElemType = null;

    this.target = target;
    this.subElemName = subElemName;
  }

  protected abstract E parseSubElemType(String type) throws CommentParserException;

  public String getTarget() {
    return target;
  }

  public E getSubElemType() {
    return subElemType;
  }

  public String getSubElemName() {
    return subElemName;
  }

  protected abstract String getTargetHref(final String definitionName, final HTMLDocumentationHelper.SourceKind sourceKind);
  protected abstract String getSubElemHref(final String definitionName, final HTMLDocumentationHelper.SourceKind sourceKind);
  protected abstract String getSubElemTypeHref(final String definitionName, final HTMLDocumentationHelper.SourceKind sourceKind);
  protected abstract String getSubElemLabel();

  @Override
  public String getReplacement(final String definitionName, final HTMLDocumentationHelper.SourceKind sourceKind) {

    //Resolve scope when it's "this"
    if(THIS_IDENTIFIER.equals(target)) {
      if(sourceKind == HTMLDocumentationHelper.SourceKind.PACKAGE) {
        throw new IllegalArgumentException("Cannot use 'this' as a scope in package documentation");
      }
      target = definitionName;
    }

    if(subElemType == null) { //link directly to the target
      return String.format("<a href=\"%s\" title=\"%s\">%s</a>",
          getTargetHref(definitionName, sourceKind),
          getTarget(), getShortName(getTarget()));
    } else {
      if(subElemName == null) {
        return String.format("<a href=\"%s\">%s</a>",
            getSubElemTypeHref(definitionName, sourceKind),
            subElemType.toString().toLowerCase());
      } else {
        return String.format("<a href=\"%s\" title=\"%s %s.%s\">%s.%s</a>",
            getSubElemHref(definitionName, sourceKind),
            getSubElemLabel(), getTarget(), getSubElemName(),
            getShortName(getTarget()), getSubElemName());
      }
    }
  }

  @Override
  public String toString() {
    return String.format("link<scope=%s,type=%s,target=%s,begin=%d,end=%d>",
        target,
        subElemType != null ? subElemType.toString().toLowerCase() : "null",
        subElemName,
        beginIndex, endIndex);
  }


}
